------------------------------------------------------------------------
--  synchro.vhd -- 
------------------------------------------------------------------------
--  Author : Mircea Dabacan
--           Copyright 2005 Digilent, Inc.
------------------------------------------------------------------------
-- This is the source file for the synchro component, 
-- provided by the Digilent Reference Component Library.

------------------------------------------------------------------------
-- Revision History:
--   01/15/2005(MirceaD): created
--   02/03/2006(MirceaD): modified
------------------------------------------------------------------------
-- Resolution:             HR(PAL=Pixels/Active Line) X VR(LAF=Lines/Active Frame) = 640X480

-- Pixel Frequency:       PF = 25MHz
--   Pixel Period:            DP = 1/DF = 40ns
--   Active Line Period:      ALP = HR X DP = 25.6us
-- Horizontal Frequency:      HF = 31.5KHz
--   Total Line Period:       TLP = 1/HF = 31.74us
--   Active Frame Period:     AFP = TLP X VR = 15.24ms
-- Vertical Frequency:        VF = 60Hz
--   Total Frame Period:      TFP = 1/VF = 16.666ms

-- Quartz Frequency:       QF = 50MHz
-- divided by 2

-- Pixel/Line Divider:      PLD = 800
-- Line/Frame Divider:      LFD = 521

-- Hcnt = Horizontal Counter
--    Counts 0 to PLD-1 (Actual Pixel Periods)
--    Generates HS=1 (Horizontal Synchro) 
--       when between HR and HR+HW
--       (HS's width = HW X ADP)
--    Generates HCurPosLAYFIELD=1 (display enable)
--       when between 64 and 320
-- HPFB = 200 (Horizontal Play Field Begin)
-- HPFE = 456 (Horizontal Play Field End)
-- HSS = Horizontal Square Size (Pixels/square)   HSS=16
-- HDispPix = Horizontal Square Counter
--   Counts 0 to HSS-1 (enabeled by HCurPosLAYFIELD) (HSS Pixels/square)
--   Generates hINT=1 when 0 (hINT = Horizontal Intensity)
--   (for each line: HSS complete cycles + begin the (HSS+1)st)
-- HSN = 16 (Horizontal Square Number)
-- HDispChar = Horizontal Displayed Character Position Counter
--   Lower 4 bits for PLAYMEM
--   Counts 0 to HSN-1 (horizontal squares)
--   incremented by hINT, enabeled by HCurPosLAYFIELD.
-- HCurPos = Horizontal Position
--   (4 bit register)


-- Vcnt =  Vertical Counter
--    Counts 0 to LFD-1 (Actual Total Line Periods)
--    Generates VS=1 (Synchro - Vertical) 
--      when between LFD and LFD+VW
--      (VS's width = VW X ATLP)
--    Generates VCurPosLAYFIELD=1 (display enable)
--      when between VPFB and VPFE
-- VPFB = 112 (Vertical Play Field Begin)
-- VPFE = 368 (Verical Play Field End)

-- VSS = 16 (Vertical Square Size) (lines/square)
-- VDispPix = Vertical Square Counter
--    Counts 0 to VSS-1 (VDispPix1 periods)
--      (enabeled by VCurPosLAYFIELD)
--    Generates vINT=1 when 0 (vINT = Vertical Intensity)
--    (for each frame: VSS complete cycles + begin the (VSS+1)st)
-- VSN = 16 (Horizontal Square Number)
-- VDispChar = Vertical Displayed Character Position Counter
--    Lower 4 bits for PLAYMEM
--    Counts 0 to VSN-1 (verical squares)
--    incremented by vINT, enabeled by VCurPosLAYFIELD.
-- VCurPos = Vertical Position
--    (4 bit register)



library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

--library work;
--use work.VGApackage.all;

entity Synchro is
generic (PLD: integer:=800; --Pixel/Line Divider
         LFD: integer:=521); --Line/Frame Divider

    port (
      ck100MHz: in std_logic;         -- system ck 
      ck25MHzOut: out std_logic;     -- VGA DAC ck 
      HS: out std_logic;             -- horizontal synchro signal               
      VS: out std_logic;             -- verical synchro signal 
      outBlank: out std_logic;              -- DAC blank signal 
      Hcnt: inout integer range 0 to PLD-1; -- horizontal counter
      Vcnt: inout integer range 0 to LFD-1  -- verical counter
      );
end Synchro;

architecture Behavioral of Synchro is
-- constants for Synchro module
constant HPW:integer:=96;   --Horizontal synchro Pulse Width (pixels)
constant HFP:integer:=16;   --Horizontal synchro Front Porch (pixels)
constant HBP:integer:=48;   --Horizontal synchro Back Porch (pixels)
constant VPW:integer:=2;    --Verical synchro Pulse Width (lines)
constant VFP:integer:=10;   --Verical synchro Front Porch (lines)
constant VBP:integer:=29;   --Verical synchro Back Porch (lines)
constant PAL:integer:=640;  --Pixels/Active Line (pixels)
constant LAF:integer:=480;  --Lines/Active Frame (lines)
--constant PLD: integer:=800; --Pixel/Line Divider
--constant LFD: integer:=521; --Line/Frame Divider

signal cntDiv: std_logic_vector(1 downto 0);  -- 2 bit counter/divider
signal ck25MHz: std_logic;  -- VGA clock

begin

divider: process (ck100MHz)   -- 2 bit counter/divider
begin
if ck100MHz'event and ck100MHz='1' then
  cntDiv <= cntDiv + "01";
end if;
end process;

  ck25MHz <= cntDiv(1);  -- dividing system clock by 2
  ck25MHzOut <= ck25MHz; -- VGA DAC ck
  outBlank <= '1' when Hcnt < PAL and Vcnt < LAF else
              '0';
syncro: process (ck25MHz)
begin

if ck25MHz'event and ck25MHz='1' then
   if Hcnt=PLD-1 then
      Hcnt<=0;
      if Vcnt=LFD-1 then Vcnt<=0;
      else Vcnt<=Vcnt+1;
      end if;
   else Hcnt<=Hcnt+1;
   end if;
   
   -- Generates HS - active high
   if Hcnt>PAL-1+HFP and Hcnt<PAL-1+HFP+HPW then 
      HS<='0';
   else 
      HS<='1';
   end if;

   -- Generates VS - active high
   if Vcnt>LAF-1+VFP and Vcnt<LAF-1+VFP+VPW then 
      VS<='0';
   else 
      VS<='1';
   end if;
end if;
end process; 

end Behavioral;
